import { DateTime } from "luxon";
import { RANGE_TYPE, WEEKDAY } from "./constants";
interface OptionsAll extends OptionsDays, OptionsMonthExact, OptionsMonthExtended, OptionsWeek {
}
export interface DateRangeMembers extends Required<OptionsAll> {
    rangeType: RANGE_TYPE;
    dateTimes: DateTime[];
    isNext: boolean;
    isPrevious: boolean;
}
export interface Offset {
    /**
     * The number of days to add or remove from the beginning of the range.
     *
     * @remarks
     * If the specified offset is positive, dates are added. If negative, dates are removed.
     *
     * @defaultValue `0`
     *
     * @example
     * ```
     * // set the reference date
     * const refDate = new Date("2020-01-17");
     *
     * // with no offset 👈
     * const month1 = new DateRange().getMonthExact({ refDate });
     * // first date in range
     * month1.dateTimes[0]; // Jan 1, 2020
     * // last date in range
     * month1.dateTimes[month1.dateTimes.length - 1]; // Jan 31, 2020
     *
     * // with positive startOffset 👈
     * const month2 = new DateRange().getMonthExact({ refDate, startOffset: 5 });
     * // first date in range
     * month2.dateTimes[0]; // Dec 27, 2020
     * // last date in range (no changes)
     * month2.dateTimes[month2.dateTimes.length - 1]; // Jan 31, 2020
     *
     * // with negative startOffset 👈
     * const month3 = new DateRange().getMonthExact({ refDate, startOffset: -5 });
     * // first date in range
     * month3.dateTimes[0]; // Jan 6, 2020
     * // last date in range (no changes)
     * month3.dateTimes[month3.dateTimes.length - 1]; // Jan 31, 2020
     * ```
     */
    startOffset?: number;
    /**
     * The number of days to add or remove from the end of the range.
     *
     * @remarks
     * If the specified offset is positive, dates are added. If negative, dates are removed.
     *
     * @defaultValue `0`
     *
     * @example
     * ```
     * // set the reference date
     * const refDate = new Date("2020-01-17");
     *
     * // with no offset 👈
     * const month1 = new DateRange().getMonthExact({ refDate });
     * // first date in range
     * month1.dateTimes[0]; // Jan 1, 2020
     * // last date in range
     * month1.dateTimes[month1.dateTimes.length - 1]; // Jan 31, 2020
     *
     * // with positive endOffset 👈
     * const month2 = new DateRange().getMonthExact({ refDate, endOffset: 5 });
     * // first date in range (no changes)
     * month2.dateTimes[0]; // Jan 1, 2020
     * // last date in range
     * month2.dateTimes[month2.dateTimes.length - 1]; // Feb 5, 2020
     *
     * // with negative endOffset 👈
     * const month3 = new DateRange().getMonthExact({ refDate, endOffset: -5 });
     * // first date in range (no changes)
     * month3.dateTimes[0]; // Jan 1, 2020
     * // last date in range
     * month3.dateTimes[month3.dateTimes.length - 1]; // Jan 26, 2020
     * ```
     */
    endOffset?: number;
}
interface RefDate {
    /**
     * The reference date to calculate the range.
     *
     * @remarks
     * Must be a Date object or luxon DateTime object.
     * Defaults to current time.
     *
     * @example
     * ```
     * // with DateTime
     * new DateRange().getWeek({ refDate: DateTime.fromISO('2023-05-15') });
     *
     * // with Date
     * new DateRange().getWeek({ refDate: new Date("2023-05-15") });
     * ```
     */
    refDate?: DateTime | Date;
}
interface RefWeekday {
    /**
     * The reference weekday to start the range from.
     *
     *@remarks
     * Must be an integer from 1 (Monday) to 7 (Sunday).
     * Defaults to Monday.
     *
     *@example
     * ```
     * // with WEEKDAY enum. The range will start from Sunday
     * new DateRange().getWeek({refWeekday: WEEKDAY.Sunday });
     * // the equivalent with a number
     * new DateRange().getWeek({ refWeekday: 7 });
     * ```
     */
    refWeekday?: WEEKDAY;
}
interface DaysCount {
    /**
     * The number of days to be included in the range generated with `getDays` method.
     */
    daysCount?: number;
}
export interface OptionsWeek extends RefDate, RefWeekday, Offset {
}
export interface OptionsDays extends RefDate, DaysCount, Offset {
}
export interface OptionsMonthExact extends RefDate, Offset {
}
export interface OptionsMonthExtended extends RefDate, RefWeekday, Offset {
}
/**
 * DateRange is the core component of easy-date-range. It provides various methods and properties for generating and handling date ranges.
 *
 * @remarks
 * To use the class, an instance must be created and initialized with one of the range generator methods.
 */
export declare class DateRange {
    /**
     * refDate of the instance.
     *
     * @remarks Default to current time.
     */
    private _refDate;
    /**
     * refWeekday of the instance.
     */
    private _refWeekday;
    /**
     * startOffset of the instance.
     */
    private _startOffset;
    /**
     * endOffset of the instance.
     */
    private _endOffset;
    /**
     * The instance dates storage.
     */
    private _dateTimes;
    /**
     * The type of a current generated date range.
     */
    private _rangeType;
    /**
     * Count of days used with getDays() method.
     */
    private _daysCount;
    /**
     * Indicates whether the DateRange is generated with a `getNext` method.
     */
    private _isNext;
    /**
     * Indicates whether the DateRange is generated with a `getPrevious` method.
     */
    private _isPrevious;
    /**
     * DateRange is an entry point for creating and storing a range of dates.
     *
     * @remarks
     * The dates can be generated with get methods.
     *
     * The constructor does not accept any parameters.
     */
    constructor();
    /**
     * The reference date for this instance.
     */
    get refDate(): DateTime;
    /**
     * The reference weekday for this instance.
     */
    get refWeekday(): WEEKDAY;
    /**
     * The array of luxon DateTimes for this instance.
     *
     * @remarks
     * To get JS Dates use `toJSDates()` method.
     */
    get dateTimes(): DateTime[];
    /**
     * The start offset for this instance.
     */
    get startOffset(): number;
    /**
     * The end offset for this instance.
     */
    get endOffset(): number;
    /**
     * The days count used with `getDays()` method for this instance.
     */
    get daysCount(): number;
    /**
     * The type of range generated for this instance.
     *
     * @remarks
     * See {@link RANGE_TYPE| the RANGE_TYPE enum} for more details.
     *
     * If `undefined`, the range has not been created yet.
     *
     */
    get rangeType(): RANGE_TYPE;
    /**
     * A boolean that indicates whether DateRange is created with a `getNext` method.
     */
    get isNext(): boolean;
    /**
     * A boolean that indicates whether DateRange is created with a `getPrevious` method.
     */
    get isPrevious(): boolean;
    /**
     * Sets or updates the instance members of the DateRange object.
     *
     * @param members - An object containing the properties of the DateRange object to be set or updated.
     * @private
     * */
    private _setMembers;
    /**
     * Checks if a given value is a valid reference date.
     *
     * @remarks A reference date can be either a JS `Date` object or a Luxon `DateTime` object.
     *
     * @param refDate - The value to check.
     * @returns True if the value is a valid reference date, false otherwise.
     */
    isValidRefDate(refDate: unknown): boolean;
    /**
     * Checks if a given value is a valid weekday (integer from 1 to 7).
     *
     * @param weekday - The value to check.
     * @returns True if weekday is an integer from 1 to 7, false otherwise.
     */
    isValidRefWeekday(weekday: unknown): boolean;
    /**
     * Checks if a given value is a valid offset (non-negative integer).
     *
     * @param offset - The value to check.
     * @returns True if the value is a non-negative integer, false otherwise.
     */
    isValidOffset(offset: unknown): boolean;
    /**
     * Returns an array of dates generated for the instance as Luxon DateTime objects.
     */
    toDateTimes(): DateTime[];
    /**
     * Returns an array of dates generated for the instance as JS Date objects.
     */
    toDates(): Date[];
    /**
     * Creates a single week range.
     *
     * @remarks
     * - By default, the method starts the range on Monday before or on the reference date and ends
     * it on Sunday after or on the reference date.
     *
     * - If not specified, the reference date is set to the current time.
     *
     * - Each date is set to the start of the day (midnight).
     *
     * @param options - An optional object to customize the date range.
     * @returns The `DateRange` with generated dates.
     *
     * @example
     *  ```
     * // get current week starting on Monday
     * const week1 = new DateRange().getWeek();
     *
     * // get week based on a refDate and starting on Sunday
     * const week2 = new DateRange().getWeek({
     * 	refDate: new Date("2023-01-10"),
     * 	refWeekday: WEEKDAY.Sunday,
     * });
     * // Generated dates:👇
     * // Sunday, January 8, 2023 at 12:00:00 AM
     * // Monday, January 9, 2023 at 12:00:00 AM
     * // Tuesday, January 10, 2023 at 12:00:00 AM
     * // Wednesday, January 11, 2023 at 12:00:00 AM
     * // Thursday, January 12, 2023 at 12:00:00 AM
     * // Friday, January 13, 2023 at 12:00:00 AM
     * // Saturday, January 14, 2023 at 12:00:00 AM
     * ```
     */
    getWeek(options?: OptionsWeek): DateRange;
    /**
     *
     * Creates a single month range extended to include the full weeks.
     *
     * @remarks
     *
     * - By default, the method starts the range on Monday before or on the first day of the month and ends
     * it on Sunday after or on the last day of the month.
     *
     * - If not specified, the reference date is set to the current time.
     *
     * - Each date is set to the start of the day (midnight).
     *
     * @param options - An optional object to customize the date range.
     * @returns The `DateRange` with generated dates.
     * @example
     *  ```
     * // Get current month extended to full weeks
     * const monthExtended1 = new DateRange().getMonthExtended();
     *
     * // Get month extended to full weeks based on a refDate and starting on Wednesday
     * const monthExtended2 = new DateRange().getMonthExtended({
     * 	refDate: new Date("2023-01-10"),
     * 	refWeekday: WEEKDAY.Wednesday,
     * });
     * // Generated dates:
     * // Wednesday, December 28, 2022 at 12:00:00 AM -> The first date of the range
     * // Thursday, December 29, 2022 at 12:00:00 AM
     * // Friday, December 30, 2022 at 12:00:00 AM
     * // ...
     * // Tuesday, January 31, 2023 at 12:00:00 AM -> The last date of the range
     * ```
     */
    getMonthExtended(options?: OptionsMonthExtended): DateRange;
    /**
     * Creates a single month range, from the first to the last day of the month.
     *
     * @remarks
     *
     * - By default, the method starts the range on the first day of the month and ends
     * it on the last day of the month.
     *
     * - If not specified, the reference date is set to the current time.
     *
     * - Each date is set to the start of the day (midnight).
     *
     * @param options - An optional object to customize the date range.
     * @returns The `DateRange` with generated dates.
     * @example
     *  ```
     * // Get current month
     * const month1 = new DateRange().getMonthExact();
     *
     * // Get month with specified refDate
     * const month2 = new DateRange().getMonthExact({
     * 	refDate: new Date("2023-01-10"),
     * });
     * // Generated dates:
     * // Sunday, January 1, 2023 at 12:00:00 AM -> The first date of the range
     * // Monday, January 2, 2023 at 12:00:00 AM
     * // Tuesday, January 3, 2023 at 12:00:00 AM
     * // ...
     * // Tuesday, January 31, 2023 at 12:00:00 AM -> The last date of the range
     *
     * // Get month with specified refDate and offsets
     * const month3 = new DateRange().getMonthExact({
     * 	refDate: new Date("2023-01-10"),
     * 	startOffset: 2,
     * 	endOffset: 2,
     * });
     * // Friday, December 30, 2022 at 12:00:00 AM  -> The range starts 2 days before default first day
     * // Saturday, December 31, 2022 at 12:00:00 AM
     * // Sunday, January 1, 2023 at 12:00:00 AM
     * // ...
     * // Tuesday, January 31, 2023 at 12:00:00 AM
     * // Wednesday, February 1, 2023 at 12:00:00 AM
     * // Thursday, February 2, 2023 at 12:00:00 AM -> The range ends 2 days after default last day
     * ```
     */
    getMonthExact(options?: OptionsMonthExact): DateRange;
    /**
     * Creates a range of custom number of days.
     *
     * @remarks
     * - The reference date is a starting point of the range.
     *
     * - If not specified, the reference date is set to the current day.
     *
     * - The length of the range can be specified with the `daysCount` property in the `options` object.
     * If not specified, the range will be created with a single date.
     *
     * - Each date is set to the start of the day (midnight).
     *
     * @param options - An optional object to customize the date range.
     * @returns The DateRange with generated dates.
     * @example
     * ```
     * // Get a current date
     * const range1 = new DateRange().getDays();
     *
     * // Get a range of 10 days starting on 2023-01-10
     * const range2 = new DateRange().getDays({
     * 	daysCount: 10,
     * 	refDate: new Date("2023-01-10"),
     * });
     * // Generated dates:
     * // January 10, 2023 at 12:00:00 AM
     * // January 11, 2023 at 12:00:00 AM
     * // January 12, 2023 at 12:00:00 AM
     * // January 13, 2023 at 12:00:00 AM
     * // January 14, 2023 at 12:00:00 AM
     * // January 15, 2023 at 12:00:00 AM
     * // January 16, 2023 at 12:00:00 AM
     * // January 17, 2023 at 12:00:00 AM
     * // January 18, 2023 at 12:00:00 AM
     * // January 19, 2023 at 12:00:00 AM
     * ```
     */
    getDays(options?: OptionsDays): DateRange;
    /**
     *
     * Generates the next range based on given one.
     *
     * @remarks
     * The method takes a DateRange object and based on its range type,
     * sets the refDate to a date that can generate the new range.
     *
     * It copies the options used to adjust the given range and applies them to the new range.
     *
     * The method shifts the refDate to the next one as follows:
     * - for a range generated with getDays, the refDate is incremented by the daysCount property.
     * - for a range generated with getWeek, the refDate is incremented by 7 days.
     * - for a range generated with getMonthExact and getMonthExtended, the refDate is set to the first day of the next month.
     *
     * In all cases, the refDate is set to the start of the day (midnight).
     *
     * To check whether the DaterRange is generated with a getNext method, access the `isNext` property of a DateRange instance.
     *
     * @param dateRange - A DateRange object with a generated range.
     * @returns The DateRange with generated dates.
     * @throws an error if DateRange is not provided or the range is not initialized.
     *
     * @example
     * ```
     * // example with a "WEEK" range
     * const week = new DateRange().getWeek({ refDate: new Date("2023-01-10") });
     * week.dateTimes; // dates from Mon, 01/09/2023 to Sun, 01/15/2023
     *
     * const weekNext = new DateRange().getNext(week);
     * weekNext.dateTimes; // dates from Mon, 01/16/2023 to Sun, 01/22/2023
     * ```
     */
    getNext(dateRange: DateRange): DateRange;
    /**  Generates the previous range based on given one.
     *
     * @remarks
     *
     * The method takes a DateRange object and based on its range type,
     * sets the refDate to a date that can generate the new range.
     *
     * It copies the options used to adjust the given range and applies them to the new range.
     *
     * The method shifts the refDate to the previous one as follows:
     *   - for a range generated with getDays, the refDate is decremented by the daysCount property.
     *   - for a range generated with getWeek, the refDate is decremented by 7 days.
     *   - for a range generated with getMonthExact and getMonthExtended, the refDate is set to the first day of the previous month.
     *
     * In all cases, the refDate is set to the start of the day (midnight).
     *
     * To check whether the DaterRange is generated with a getPrevious method, access the `isPrevious` property of a DateRange instance.
     *
     * @param dateRange - A DateRange object with a generated range range.
     * @returns The DateRange object with a generated range of the same type and options as the given one but with shifted dates.
     * @throws an error if DateRange is not provided or the range is not initialized.
     *
     * @example
     * ```
     * // example with a "WEEK" range
     * const week = new DateRange().getWeek({ refDate: new Date("2023-01-10") });
     * week.dateTimes; // dates from Mon, 01/09/2023, to Sun, 01/15/2023
     *
     * const weekPrevious = new DateRange().getPrevious(week);
     * weekPrev.dateTimes; // dates from Mon, 01/02/2023 to Sun, 01/08/2023
     * ```
     */
    getPrevious(dateRange: DateRange): this;
}
export {};
