declare class CalendarGridError extends Error {
    code: string;
    /**
     * Creates a new instance of CalendarRangeError.
     *
     * @param code - The {@link ErrorCode} of the error.
     * @param message - The message of the error.
     */
    constructor(code: string, message: string);
}
export declare class InvalidParameterError extends CalendarGridError {
    paramName: string;
    paramValue: unknown;
    expected: string;
    description?: string;
    /**
     * Creates a new instance of InvalidParameterError.
     *
     * @param paramName - The name of the invalid parameter.
     * @param paramValue - The value of the invalid parameter.
     * @param expected - The expected type or format of the parameter value.
     * @param description - An optional description of the validation error.
     *
     * @example
     * ```
     * // create an instance of InvalidParameterError with some sample parameters
     *  const error = new InvalidParameterError(
     * 	"foo",
     * 	42,
     * 	"string",
     * 	"This option is required for the operation.",
     * );
     * // log the error to the console
     * console.log(error)
     * //-> Error: The value of the foo is invalid. You passed 42, but string is expected. This option is required for the operation.
     * ```
     */
    constructor(paramName: string, paramValue: unknown, expected: string, description?: string);
}
export declare class MissingArgumentError extends CalendarGridError {
    /**
     * The name of the missing argument.
     */
    paramName: string;
    /**
     * The name of the function where the error occurs.
     */
    funcName: string;
    /**
     * Represents an error that occurs when an argument is not provided but required in the function.
     * @param paramName - The name of the missing argument.
     * @param funcName - The name of the function where the error occurs.
     *
     * @example
     * ```
     * throw new MissingArgumentError("dateRange", "DateRange.getNext()")
     * // -> Error: Missing dateRange argument in DateRange.getNext() function.
     * ```
     */
    constructor(paramName: string, funcName: string);
}
export declare class EmptyDateRangeError extends CalendarGridError {
    funcName: string;
    /**
     * @example
     * ```
     * throw new EmptyDateRangeError("getNext method");
     * // -> Error: The DateRange object you have passed to the getNext method is empty. You need to create a range using one of the get methods first'.
     * ```
     */
    constructor(funcName: string);
}
export declare class InvalidStartOffsetError extends InvalidParameterError {
    /**
     * Represents an error that occurs when the startOffset parameter is invalid.
     * @param paramValue - The value of the startOffset parameter that caused the error.
     */
    constructor(paramValue: unknown);
}
export declare class InvalidEndOffsetError extends InvalidParameterError {
    /**
     * Represents an error that occurs when the endOffset parameter is invalid.
     * @param paramValue - The value of the endOffset parameter that caused the error.
     */
    constructor(paramValue: unknown);
}
export declare class InvalidRefWeekdayError extends InvalidParameterError {
    /**
     * Represents an error that occurs when the refWeekday parameter is invalid.
     * @param paramValue - The value of the refWeekday parameter that caused the error.
     */
    constructor(paramValue: unknown);
}
export declare class InvalidRefDateError extends InvalidParameterError {
    /**
     * Represents an error that occurs when the refDate parameter is invalid.
     * @param paramValue - The value of the refDate parameter that caused the error.
     */
    constructor(paramValue: unknown);
}
export declare class InvalidDateRangeError extends InvalidParameterError {
    /**
     * Represents an error that occurs when the dateRange parameter is invalid.
     * @param paramValue - The value of the dateRange parameter that caused the error.
     *
     * @example
     * ```
     * throw new InvalidDateRangeError(4)
     * // -> Error: The value of the dateRange parameter is invalid. You passed 4, but a valid, non empty DateRange object is expected.
     * ```
     */
    constructor(paramValue: unknown);
}
export {};
